package rocketapi.sp.auth;

import java.util.Arrays;
import java.util.Map;
import java.util.Optional;

import javax.servlet.http.HttpServletRequest;

import org.apache.tomcat.util.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.github.alenfive.rocketapi.entity.vo.LoginVo;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jws;
import io.jsonwebtoken.Jwts;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import rocketapi.sp.config.Config;

/**
 * 登录工具类
 * 
 * @apiNote 覆盖默认功能。使用oauth2令牌。
 */
@Slf4j
@Component
public class JwtTokenLoginService extends com.github.alenfive.rocketapi.service.LoginService {

	RestTemplate restTemplate = new RestTemplate();

	public JwtTokenLoginService() {
		log.debug("create Oauth2TokenLoginService");
	}

	/**
	 * 取得登录用户：解析 jwt 令牌
	 * 
	 * @param request
	 */
	@Override
	public String getUser(HttpServletRequest request) {
		log.debug("start getUser");

		// 取得客户端提供的令牌原值
		String tokenString = request.getHeader(config.getHeaderTokenKey());

		// 解析令牌字符串
		JwtUser user = getUserFromJwt(tokenString);

		if (user == null) {
			return null;
		}
		return user.getCode();
	}

	/**
	 * 登录：生成token
	 */
	@Override
	public String getToken(LoginVo loginVo) throws JsonProcessingException {
		log.debug("start getToken");
		String url = config.getLoginUrl();
		ResponseEntity<AuthResponse> auth = restTemplate.postForEntity(url, loginVo, AuthResponse.class);
		if (auth.getStatusCode().is2xxSuccessful()) {
			// 登录成功
			return auth.getBody().getToken();
		}
		// 登录失败
		return null;
	}

	@Setter
	@Getter
	public static class AuthResponse {
		private String userInfo;
		private String token;
	}

	/**
	 * 从 jwt 令牌解析用户信息
	 * 
	 * @param tokenString
	 * @return
	 */
	public JwtUser getUserFromJwt(String tokenString) {
		if (StringUtils.isEmpty(tokenString)) {
			log.debug("token is empty");
			return null;
		}
		JwtUser user = exactUser(tokenString);
		if (user == null) {
			log.debug("token is invalid ({})", tokenString);
			return null;
		}
		return user;
	}

	@Autowired
	Config config;

	@SuppressWarnings({ "unchecked" })
	private JwtUser exactUser(String jwt) {
		Jws<Claims> claims = null;

		try {
			String secretKey = Base64.encodeBase64String(config.getKey().getBytes());
			claims = Jwts.parser().setSigningKey(secretKey).parseClaimsJws(jwt);
		} catch (Exception ex) {
			log.error("exact token error", ex);
			return null;
		}
		Claims body = claims.getBody();
		JwtUser user = new JwtUser();

		Map<String, Object> userMap = (Map<String, Object>) body.get(config.getClaimUserKey());
		user.setCode((String) userMap.get(config.getUsernameKey()));
		user.setName((String) userMap.get(config.getNicknameKey()));

		Map<String, Object> metaMap = (Map<String, Object>) userMap.get(config.getMetadataKey());
		user.setAvator((String) metaMap.get(config.getAvatarKey()));
		String authorityString = Optional.ofNullable((String) metaMap.get(config.getAuthoritiesKey())).orElse("");
		user.setAuthorities(Arrays.asList(authorityString.split(",")));

		// 其他数据
		metaMap.remove(config.getAvatarKey());
		metaMap.remove(config.getAuthoritiesKey());
		user.setMetadata(metaMap);

		return user;
	}
}
